
/*          
           Joahermen = Joaquim Herdy Menezes
               www.realjoia.net/arduino/
                     <- 2022G01 ->
            Painel Central DCC WiFi RealJoia
                    Versao ifthen
                   EEPROM  Teclado
               WeMos D1 Mini Esp 8266
             Pinos PCA9685 Servo Motores
        SDA no SDA Pino D2 e SCL no SCL Pino D1 
                 Fios do Servo Motor
    Marrom = Gnd | Vermelho = Vcc | Laranja = PWM 
     Preto = Gnd | Vermelho = Vcc | Branco = PWM
*/

#include "Arduino.h"                                                // Biblioteca Master
#include "Wire.h"                                                   // Biblioteca i2C 
#include <WiFiClient.h>                                             // Cliente WiFi
#include <ESP8266WiFi.h>                                            // Biblioteca ESP8266
#include <ESP8266WebServer.h>                                       // Servidor Web
#include <LiquidCrystal_I2C.h>                                      // Display LCD 
#include <SoftwareSerial.h>                                         // Serial UART Tx Rx
#include <EEPROM.h>                                                 // Habilita Memoria Flash do ESP8266
#include <MUX74HC4067.h>                                            // Multiplexador de Sinais Analogicos e Digitais
#include <Adafruit_PWMServoDriver.h>                                // ServoMotor 

SoftwareSerial SerialWiFi(3,1); 
ESP8266WebServer Server(51623);                                     // Define a Porta de acesso da Pagina HTML
LiquidCrystal_I2C lcd(0x3f,20,4);                                   // Define o LCD i2C com 20 colunas e 4 linhas
WiFiClient Client; 
Adafruit_PWMServoDriver ServoLeds = Adafruit_PWMServoDriver(0x40);  // Configura o PCA9685

#define HostiP "192.168.4.1"                                        // Define o Enderecso de acesso da Pagina HTML 

#define Hertz 60                                                    // Frequencia Ciclos por Segundo do ServoMotor

#ifndef APSSID
#define APSSID "wifijoia"                                           // Define Login da rede WiFi
#define APPSK  "WiFiJoia"                                           // Define Senha da rede WiFi
#endif

const char *ssid = APSSID;
const char *password = APPSK;

IPAddress staticIP(192,168,4,134); 
IPAddress dns(192,168,4,1); 
IPAddress gateway(192,168,4,1); 
IPAddress subnet(255,255,255,0); 

String NossoIP = "192.168.4.1:51623";                               // Enderecso iP com a Porta da Rede WiFi

unsigned int PulMin = 140;                                          // Seta Pulso minimo para posicsao 0°
unsigned int PulMax = 560;                                          // Seta Pulso maximo para posicsao 180°
                                                                    // Pinos WeMos ESP8266  -  Funsoes      -  Botoes
const int Botinp0 = 2;                                              // D4 - Nivel PullUp    -  Opsoes       -  Branco DirBax
const int Botinp3 = 15;                                             // D8 - Nivel PullDown  -  AMV          -  Preto
const int Botinp2 = 12;                                             // D6 - Nivel Ambos     -  Avante       -  Verde
const int Botinp8 = 13;                                             // D7 - Nivel Ambos     -  Recua        -  Amarelo
const int Botinp5 = 14;                                             // D5 - Nivel PullUp    -  Para         -  Vermelho
const int Botinp6 = 0;                                              // D3 - Nivel Ambos     -  incrementa   -  Azul
const int Botinp4 = 16;                                             // D0 - Nivel PullUp    -  Decrementa   -  Branco

// int Total;
// int Comando; 
// String Letra = "0"; 
// String Recebido = "";
String AMVSouTrem = "Trem";
char Palavra[19];
char Caracter;
String Leitura = "";
int Memoria; 
int EpromAMVS = 0;                                                  // Posicsao na EEPROM para AMV + Graus + Semafaros + Etc de 500-999 
int EpromTrem = 0;                                                  // Posicsao na EEPROM Grupo Trens de 0-99 + 100-199 e Efeitos de 200-299
int SMTouAMV = 0;
byte ByteAlto; 
byte ByteBaxo; 
int VaiVem; 
int Valor;
String StriChar = "PainelCentralDCC";
String RotaTrem = "|Y|Y|Y|Y|Y|Y|Y|Y|Y";                             // AMV Linha Principal "|" ou Secundaria "Y"; 
String ZigZag = "";                                                 // Uzo geral
String LuzSom = "0";                                                // Efeitos
String MeuTrem = "3";                                               // Numero da Locomotiva 
String Ordem = "<0>";                                               // Desativa Central DCC 
String AMVSM$ = "SMT";                                              // SMT ou AMV 
String Acsao$ = "1";                                                // Acionamento da Locomotiva 
String Ativa$ = ""; 
String BitVe$ = "00000000";
String Efeix$ = "<f 3 0>";                                          // Comando Efeitos 
String Extra$ = "<y 0 0>";                                          // Comando AMV ou SMT
String Kodin$ = "0";
String LayKa$ = "1111111111"; 
String Opsao$ = "Trem";                                             // Seletor - Trem - Efex - Slet - Prog -
String Panel$ = "}->";
String Ramal$ = "0";                                                // Referente ao AMV ou SMT  
String Legal$ = "<t 0 3 0 1>";                                      // Comando Central DCC RealJoia <Funsao Ramal Locomotiva Velocidade Sentido>
String Veloz$ = "0";                                                // Velociadade da Locomotiva 
String Vizor$ = "|!|";                                              // Mostra Comando Ativo no Display (> Parar <)
char Tecla = ' ';
long QualTrem = 0;
int Atual = 5;
int Bytes = 0;
int Numero = 0;
int Entre = 1;
int Final = 10;
int Graus = 10; 
int Ponto = 1;
int Testar = 0;
int Xecar = 1; 
int Ycogn = 0; 
unsigned long Tempo1 = 0;
int SeletSM = 0; 
long NumLoc[10];                                                    // Numeros das Locomotivas                            - 2 Bytes x 10 
long QuePAR[10];                                                    // Acsao Para_Avante_Retorn atual de cada Locomotiva  - 1 Byte x 10 
long VeLoco[10];                                                    // Velocidade atual de cada Locomotiva                - 1 Byte x 10 
long FunLoc[10];                                                    // Funsao Luz Som etc                                 - 1 Byte x 10 
int Desvios[16];
int RumosSM[16];                                                    // - 1 Bytes x 16
int AtualSM[16];                                                    // Posicsao Atual da Acsao de cada MultiFunsao        - 1 Bytes x 6 
int BasiqSM[16];                                                    // Limites minimos da rotacsao de cada MultiFunsao    - 1 Bytes x 6 
int LimitSM[16];                                                    // Limites maximos da rotacsao de cada MultiFunsao    - 1 Bytes x 6 
int RampaSM[16];                                                    //                                                    - 1 Bytes x 6 

const String postForms = "<!DOCTYPE html> <html> <head> <meta charset='UTF-8'> <title>CelularDCC RealJoia</title> <script> var Bytes = 0; var Conta = 0; var Dados = '0'; var Flags = '0'; var Geral = '0'; var Horbi = ' 1>'; var image = ''; var Klesh = '0'; var Layka = ''; var Maqna = '3'; var Outra = '0000000000'; var Quale = 'NomeTrem'; var Rotas = '<t 0 '; var Sinal = 'Vai'; var Total = 0; var Veloz = '64'; var Xekar = '0'; var Warey = '0000000000'; const RealWiFi = 'http://192.168.4.1:51623'; </script> <style> .Tabela { width: 1000px; height: 1600px; } .Figura { width: 960px; height: 540px; } .Ordem { width: 50%; height: 100px; font-size: 40pt; padding: 4px; margin: 0px; border: 1px solid rgb(0, 0, 0, 0.14);} .nomcod { width: 98%; height: 130px; font-size: 48pt; padding: 4px; margin: 4px; border: 1px solid rgb(0, 0, 0, 0.14);} .texto { width: 95%; height: 130px; font-size: 48pt; padding: 4px; margin: 4px; border: 1px solid rgb(0, 0, 0, 0.14);} .Dimag { width: 96%; height: 80px; font-size: 36pt; padding: 4px; margin: 4px; border: 1px solid rgb(0, 0, 0, 0.14);} .Setor { width: 100%; height: 144px; font-size: 48pt; margin: 2px; border: 1px solid rgb(0, 0, 0, 0.400); background-color: rgb(255, 175, 0); cursor: pointer; transition: 100ms;} .Setor:hover { background-color: rgb(255, 255, 255, 0.300);} .botao { width: 30%; height: 180px; font-size: 48pt; margin: 2px; border: 1px solid rgb(0, 0, 0, 0.400); background-color: rgb(255, 175, 0); cursor: pointer; transition: 100ms;} .botao:hover { background-color: rgb(255, 255, 255, 0.300);} .butom { width: 9%; height: 144px; font-size: 48pt; margin: 0px; border: 1px solid rgb(0, 0, 0, 0.400); background-color: rgb(255, 175, 0); cursor: pointer; transition: 100ms;} .butom:hover { background-color: rgb(255, 255, 255, 0.300);} .dlocom { width: 14%; height: 144px; font-size: 48pt; padding: 4px; margin: 4px; border: 1px solid rgb(0, 0, 0, 0.400); background-color: rgb(255, 175, 0); cursor: pointer; transition: 100ms;} .dlocom:hover { background-color: rgb(255, 175, 0, 0.300);} </style> </head> <body bgcolor='#000000' text='#FFFFFF' link='#00FFFF' vlink='#FFFF00' alink='#00FF00' bgproperties='Fixed' resizable='no' leftmargin='0' topmargin='0' marginwidth='0' marginheight='0'> <center> <table class='Tabela'><tr valign='top'><td align='center'> <div id='Trens' style='display:none;'> <table class='Tabela' bgcolor='#0000FF'><tr valign='top'><td colspan='3' align='center'> <h1>PainelDCC RealJoia</h1> </td></tr><tr><td width='56%'> <input type='text' class='nomcod' id='Nome9'><br> <input type='text' class='nomcod' id='Nome8'><br> <input type='text' class='nomcod' id='Nome7'><br> <input type='text' class='nomcod' id='Nome6'><br> <input type='text' class='nomcod' id='Nome5'><br> <input type='text' class='nomcod' id='Nome4'><br> <input type='text' class='nomcod' id='Nome3'><br> <input type='text' class='nomcod' id='Nome2'><br> <input type='text' class='nomcod' id='Nome1'><br> </td><td width='28%'> <input type='text' class='texto' id='Trem9'><br> <input type='text' class='texto' id='Trem8'><br> <input type='text' class='texto' id='Trem7'><br> <input type='text' class='texto' id='Trem6'><br> <input type='text' class='texto' id='Trem5'><br> <input type='text' class='texto' id='Trem4'><br> <input type='text' class='texto' id='Trem3'><br> <input type='text' class='texto' id='Trem2'><br> <input type='text' class='texto' id='Trem1'><br> </td><td width='16%'> <input type='button' class='Setor' onclick='Maquin9()' value='9'><br> <input type='button' class='Setor' onclick='Maquin8()' value='8'><br> <input type='button' class='Setor' onclick='Maquin7()' value='7'><br> <input type='button' class='Setor' onclick='Maquin6()' value='6'><br> <input type='button' class='Setor' onclick='Maquin5()' value='5'><br> <input type='button' class='Setor' onclick='Maquin4()' value='4'><br> <input type='button' class='Setor' onclick='Maquin3()' value='3'><br> <input type='button' class='Setor' onclick='Maquin2()' value='2'><br> <input type='button' class='Setor' onclick='Maquin1()' value='1'><br> </td></tr><tr valign='bottom'><td align='right' colspan='3'> <font size='4' color='#003FFF'><i>Joahermen</i></font><br> <input type='submit' class='botao' id='Babay' value='Babay' onclick='Babay()'> <input type='submit' class='botao' id='Ligar' value='Ligar' onclick='Ligar()'> <input type='button' class='dlocom' onclick='Exibir2()' value='}' /> </td></tr></table> </div> </td><td> <div id='Central' style='display:inline;'> <table class='Tabela' bgcolor='#FFCF00'><tr valign='top'><td align='center'> <form method='post' id='Urgent' name='Urgent' action='http://192.168.4.1:51623/ordem/'> <input type='text' class='Ordem' id='Ordem' name='Ordem'> <input type='submit' class='botao' id='Atuar' name='Atuar' value='Ordem'></form> <p><img class='Figura' id='Verimag' src=''></p> <input type='button' class='botao' id='Atraz' value='< = {' onclick='Atraz()'> <input type='submit' class='botao' id='Parar' value='| - |' onclick='Parar()'> <input type='button' class='botao' id='Frent' value='} = >' onclick='Frent()'> <p><input type='submit' class='botao' id='Menor' value='> - <' onclick='Menor()'> <input type='submit' class='botao' id='Maior' value='< + >' onclick='Maior()'></p> <input type='button' class='butom' id='Byte9' value='9' onclick='BitVal9()'> <input type='button' class='butom' id='Byte8' value='8' onclick='BitVal8()'> <input type='button' class='butom' id='Byte7' value='7' onclick='BitVal7()'> <input type='button' class='butom' id='Byte6' value='6' onclick='BitVal6()'> <input type='button' class='butom' id='Byte5' value='5' onclick='BitVal5()'> <input type='button' class='butom' id='Byte4' value='4' onclick='BitVal4()'> <input type='button' class='butom' id='Byte3' value='3' onclick='BitVal3()'> <input type='button' class='butom' id='Byte2' value='2' onclick='BitVal2()'> <input type='button' class='butom' id='Byte1' value='1' onclick='BitVal1()'> <input type='button' class='butom' id='Byte0' value='0' onclick='BitVal0()'> </td></tr><tr valign='bottom'><td align='right'> <center><input type='text' class='Dimag' id='Dimag'></center> <input type='button' class='botao' id='Salvar' value='Salvar' onclick='Salvimg()'> <input type='button' class='botao' id='Captar' value='Captar' onclick='Captar()'> <input type='button' class='dlocom' onclick='Exibir1()' value='{' /> </td></tr></table> </div> </td></tr></table> <script> function Captar(){ for(Conta = 1; Conta <= 9; Conta++){ Xekar = Conta.toString(); Veloz = localStorage.getItem('Veloz' + Xekar); if (Veloz != null){ Rotas = localStorage.getItem('Rotas' + Xekar); Maqna = localStorage.getItem('Trem' + Xekar); Horbi = localStorage.getItem('Horbi' + Xekar); Warey = localStorage.getItem('Warey' + Xekar); Klesh = localStorage.getItem('Klesh' + Xekar); Quale = localStorage.getItem('Nome' + Xekar); image = localStorage.getItem('image' + Xekar); document.getElementById('Trem' + Xekar).value = Maqna; document.getElementById('Nome' + Xekar).value = Quale; document.getElementById('Dimag').value = image; document.getElementById('Verimag').src = image; } else { Rotas = '<t ' + Xekar + ' '; Maqna = Xekar; Veloz = '0'; Horbi = ' 1>'; Quale = 'Locomotiva' + Xekar; Warey = '0000000000'; Klesh ='0'; image = ''; localStorage.setItem('Rotas' + Xekar, Rotas); localStorage.setItem('Trem' + Xekar, Maqna); localStorage.setItem('Veloz' + Xekar, Veloz); localStorage.setItem('Horbi' + Xekar, Horbi); localStorage.setItem('Warey' + Xekar, Warey); localStorage.setItem('Klesh' + Xekar, Klesh); localStorage.setItem('Nome' + Xekar, Quale); localStorage.setItem('image' + Xekar, image); document.getElementById('Trem' + Xekar).value = Maqna; document.getElementById('Nome' + Xekar).value = Quale; Sinal = 'Okey';} Mostrar();}} function Maquin1(){Xekar = '1'; Komando(); Exibir2();} function Maquin2(){Xekar = '2'; Komando(); Exibir2();} function Maquin3(){Xekar = '3'; Komando(); Exibir2();} function Maquin4(){Xekar = '4'; Komando(); Exibir2();} function Maquin5(){Xekar = '5'; Komando(); Exibir2();} function Maquin6(){Xekar = '6'; Komando(); Exibir2();} function Maquin7(){Xekar = '7'; Komando(); Exibir2();} function Maquin8(){Xekar = '8'; Komando(); Exibir2();} function Maquin9(){Xekar = '9'; Komando(); Exibir2();} function Komando() { if (Veloz == null){return;} Rotas = '<t ' + Xekar + ' '; Maqna = document.getElementById('Trem' + Xekar).value; Quale = document.getElementById('Nome' + Xekar).value; localStorage.setItem('Rotas' + Xekar, Rotas); localStorage.setItem('Trem' + Xekar, Maqna); localStorage.setItem('Nome' + Xekar, Quale); Mostrar();} function Salvimg(){ image = document.getElementById('Dimag').value; localStorage.setItem('image' + Xekar, image); document.getElementById('Verimag').src = image;} function Mostrar(){ Rotas = localStorage.getItem('Rotas' + Xekar); Maqna = localStorage.getItem('Trem' + Xekar); Veloz = localStorage.getItem('Veloz' + Xekar); Horbi = localStorage.getItem('Horbi' + Xekar); Quale = localStorage.getItem('Nome' + Xekar); image = localStorage.getItem('image' + Xekar); Klesh = localStorage.getItem('Klesh' + Xekar); Geral = Rotas + Maqna + ' ' + Veloz + Horbi; document.getElementById('Ordem').value = Geral; document.getElementById('Dimag').value = image; document.getElementById('Verimag').src = image; } function Ligar(){ document.getElementById('Ordem').value = '<1>'; document.getElementById('Urgent').submit();} function Babay(){ document.getElementById('Ordem').value = '<0>'; document.getElementById('Urgent').submit();} function Menor(){ if (Sinal == 'Okey'){return;} else { Veloz = localStorage.getItem('Veloz' + Xekar); Conta = parseInt(Veloz); if (Conta > 7){Conta = Conta - 8}; Veloz = Conta.toString(); localStorage.setItem('Veloz' + Xekar, Veloz); Mostrar(); document.getElementById('Urgent').submit();}} function Maior(){ if (Sinal == 'Okey'){return;} else { Veloz = localStorage.getItem('Veloz' + Xekar); Conta = parseInt(Veloz); if (Conta < 116){Conta = Conta + 8}; Veloz = Conta.toString(); localStorage.setItem('Veloz' + Xekar, Veloz); Mostrar(); document.getElementById('Urgent').submit();}} function Atraz(){if (Sinal != 'Vai'){Sinal = 'Vem'; Horbi = ' 0>'; localStorage.setItem('Horbi' + Xekar, ' 0>'); Mostrar(); document.getElementById('Urgent').submit();} else {return;}} function Parar(){Sinal = 'Okey'; Geral = Rotas + Maqna + ' 0' + Horbi; document.getElementById('Ordem').value = Geral; document.getElementById('Urgent').submit();} function Frent(){if (Sinal != 'Vem'){Sinal = 'Vai'; Horbi = ' 1>'; localStorage.setItem('Horbi' + Xekar, ' 1>'); Mostrar(); document.getElementById('Urgent').submit();} else {return;}} function BitVal9(){Conta = 9; Bytes = 512; Calculo();} function BitVal8(){Conta = 8; Bytes = 256; Calculo();} function BitVal7(){Conta = 7; Bytes = 128; Calculo();} function BitVal6(){Conta = 6; Bytes = 64; Calculo();} function BitVal5(){Conta = 5; Bytes = 32; Calculo();} function BitVal4(){Conta = 4; Bytes = 16; Calculo();} function BitVal3(){Conta = 3; Bytes = 8; Calculo();} function BitVal2(){Conta = 2; Bytes = 4; Calculo();} function BitVal1(){Conta = 1; Bytes = 2; Calculo();} function BitVal0(){Conta = 0; Bytes = 1; Calculo();} function Calculo(){ Warey = localStorage.getItem('Warey' + Xekar); Klesh = localStorage.getItem('Klesh' + Xekar); Geral = Warey.split(\"\").reverse().join(\"\"); Total = parseInt(Klesh); Flags = Geral.substring(Conta, Conta + 1); if(Flags == '1'){Flags = '0'; if (Total - Bytes >= 0){Total = Total - Bytes;}} else if(Flags == '0'){Flags = '1'; if (Total + Bytes <= 1023){Total = Total + Bytes;}} Outra = Geral.substring(0, Conta) + Flags + Geral.substring(10, Conta +1); Klesh = Total.toString(); Warey = Outra.split(\"\").reverse().join(\"\"); localStorage.setItem('Warey' + Xekar, Warey); localStorage.setItem('Klesh' + Xekar, Klesh); Geral = '<f ' + Maqna + ' ' + Klesh + '>'; document.getElementById('Ordem').value = Geral; document.getElementById('Urgent').submit();} function Exibir1(){ document.getElementById('Central').style.display='none'; document.getElementById('Trens').style.display ='inline';} function Exibir2(){ document.getElementById('Trens').style.display='none'; document.getElementById('Central').style.display ='inline';} </script> </td></tr></table> </center> </body> </html>";

void setup() {
    Server.begin();
    Wire.begin();
    ServoLeds.begin();  
    EEPROM.begin(1200); 
    Serial.begin(9600);  
    SerialWiFi.begin(9600);
    WiFi.disconnect(); 
    WiFi.mode(WIFI_STA); 
    WiFi.config(staticIP, dns, gateway, subnet);
    WiFi.softAP(ssid, password);
    Server.on("/", handleClient);
    Server.on("/ordem/", handleOrdem);
    pinMode(Botinp0, INPUT); 
    pinMode(Botinp2, INPUT); 
    pinMode(Botinp3, INPUT); 
    pinMode(Botinp4, INPUT); 
    pinMode(Botinp5, INPUT); 
    pinMode(Botinp6, INPUT); 
    pinMode(Botinp8, INPUT); 
    lcd.init(); 
    lcd.backlight();
    CaptaTrem();
    Mensagem();
    ServoLeds.setPWMFreq(Hertz);                                                    // Frequencia PWM de 60Hz o T=16,66ms
    for (VaiVem = 0; VaiVem <= 15; VaiVem++) { 
          ServoLeds.setPWM(VaiVem, 0, 360);                                         // Seta todos canais dos Servo Motores 
    } 
    for (VaiVem = 0; VaiVem < 10; VaiVem++) { 
          VeLoco[VaiVem] = 0;                                                       // ReSeta Velocidade de todas Locomotivas
          QuePAR[VaiVem] = 1;                                                       // Seta Avante para todas Locomotivas
    }
    SerialWiFi.println("");                                                         // NumLoc[10] Locomotiva Exclusiva do Painel de Controle
    SerialWiFi.println("<1>"); 
    SerialWiFi.flush();                                                             // Coordena Comando para Central DCC
    delay(9000);
    lcd.clear(); 
    lcd.setCursor(0,0);
    TremPainel(); 
}

void handleOrdem() {
  Server.send(204, "");
  if (Server.args() > 0 ) {
  for ( uint8_t VaiVem = 0; VaiVem < Server.args(); VaiVem++ ) {
  Ordem = Server.argName(VaiVem); 
  if (Server.argName(VaiVem) == "Ordem") {
  Ativa$ = Server.arg(VaiVem); 
  if (Ativa$.substring(1, 2) != "y") {
  SerialWiFi.println(Server.arg(VaiVem)); 
  SerialWiFi.flush();
  } else { 
    SeletorAMV();   
  }
   }
  }
 }
}

void handleClient() {
if (Server.args() > 0 ){
  for ( uint8_t VaiVem = 0; VaiVem < Server.args(); VaiVem++ ) {
  Ordem = Server.argName(VaiVem); 
if (Server.argName(VaiVem) == "plain") {
  Ativa$ = Server.arg(VaiVem);
  SerialWiFi.println(Server.arg(VaiVem)); 
  SerialWiFi.flush();
  return;
   }
  }
} else {
  Server.send(200, "text/html", postForms); 
 }
}

void loop() {
  if (SerialWiFi.available() > 0) {
    Bibliotek();
  }
    Konsole();     
  Server.handleClient(); 
  delay(10);
}

void Bibliotek() {
      Caracter = SerialWiFi.read(); 
if (Caracter == 94) {                                                               // Para rezetar Todos Locais da EEPROM digitar '^'
      Caracter = 0; 
  for (VaiVem = 0; VaiVem < 1000; VaiVem++) {                                       // Para preservar a EEPROM soh grava Dado se for diferente do jah existente
    if (EEPROM.read(VaiVem != Caracter)) {
      EEPROM.write(VaiVem, Caracter); 
      EEPROM.commit();
    }
      SerialWiFi.print(" Local_EEPROM = ");
      SerialWiFi.print(VaiVem);      
      SerialWiFi.print(" - Numero = ");
      SerialWiFi.println(Memoria);      
  }
}
if (Caracter == 59) {                                                               // Para Selecionar Local da EEPROM para Ler ou Gravar digitar ';' & 'Localizacsao' 
      EpromTrem = SerialWiFi.parseInt();                                            // na memoria da EEPROM 
      SerialWiFi.println();
      SerialWiFi.print(" EpromTrem = ");
      SerialWiFi.println(EpromTrem);      
      SerialWiFi.println();
}
if (Caracter == 58) {                                                               // Para gravar na EEPROM digitar ':' & 'Numero pretendido ateh o maximo' 
      Memoria = SerialWiFi.parseInt(); 
    if (EpromTrem >= 200 && EpromTrem <= 1199) {
    if (Memoria >= 0 && Memoria < 256) {
  if (EEPROM.read(EpromTrem != Memoria)) {
      EEPROM.write(EpromTrem, Memoria); 
      EEPROM.commit();
    }
      SerialWiFi.println();
      SerialWiFi.print(" Local_EEPROM = ");
      SerialWiFi.println(EpromTrem);      
      SerialWiFi.print(" Valor = ");
      SerialWiFi.println(Memoria);      
      SerialWiFi.println();
    }
  }
}
if (Caracter == 61) {                                                               // Para gravar na EEPROM digitar '=' & 'Numero pretendido ateh o maximo' 
      Memoria = SerialWiFi.parseInt(); 
  if (EpromTrem >= 0 && EpromTrem <= 99) {
    if (Memoria >= 0 && Memoria <= 9999) {
      ByteAlto = Memoria / 256;                                                     // Valor Maior no enderecso Alto da EEPROM
      ByteBaxo = Memoria;                                                           // Valor Menor no enderecso Baixo da EEPROM
  if (EEPROM.read((EpromTrem + 100) != ByteAlto)) {
      EEPROM.write((EpromTrem + 100), ByteAlto); 
      EEPROM.commit();
  }
  if (EEPROM.read(EpromTrem != ByteBaxo)) {
      EEPROM.write(EpromTrem, ByteBaxo); 
      EEPROM.commit();
  }
      SerialWiFi.println("");
      SerialWiFi.print(" LocMemAlta = ");
      SerialWiFi.print(EpromTrem + 100);                                            // Mostra o enderecso Alto
      SerialWiFi.print(" - Valor = ");
      SerialWiFi.print(ByteAlto);                                                   // Mostra o Valor Maior em Hexadecimal 
      SerialWiFi.print(" LocMemBaixa = ");
      SerialWiFi.print(EpromTrem);                                                  // Mostra o enderecso Baixo
      SerialWiFi.print(" - Valor = ");
      SerialWiFi.print(ByteBaxo);                                                   // Mostra o Valor Menor em Hexadecimal
      SerialWiFi.println(" ");
    }
  }
}
if (Caracter == 62) {                                                               // Para Setar incrementando Bloco da EEPROM digitar '>'
  if (AMVSouTrem == "Trem") {                                                       // Seta incremento EpromTrem na EEPROM para Grupo de Trens e Efeitos
  if (EpromTrem < 90) { EpromTrem = EpromTrem + 10;
  } else { 
    EpromTrem = 90;
  }
  } else {                                                                  // Seta incremento EpromAMVS na EEPROM para Grupo AMV Graus Semafaros Etc
  if (EpromAMVS < 80) { EpromAMVS = EpromAMVS + 16;
  } else { 
    EpromAMVS = 80;
  }
  }
      SerialWiFi.println();
      SerialWiFi.print(" - EpromTrem = ");
      SerialWiFi.print(EpromTrem);      
      SerialWiFi.print(" - EpromAMVS = ");
      SerialWiFi.print(EpromAMVS);      
      SerialWiFi.println();
}
if (Caracter == 60) {                                                               // Para Setar decrementando Bloco da EEPROM digitar '<'
  if (AMVSouTrem == "Trem") {                                               // Seta decremento EpromTrem na EEPROM para Grupo de Trens e Efeitos
  if (EpromTrem > 9) { EpromTrem = EpromTrem - 10;
  } else { 
    EpromTrem = 0;
  }
  } else {                                                                  // Seta decremento EpromAMVS na EEPROM para Grupo AMV Graus Semafaros Etc
  if (EpromAMVS > 15) { EpromAMVS = EpromAMVS - 16;
  } else { 
    EpromAMVS = 0;
  }
  }
      SerialWiFi.println();
      SerialWiFi.print(" - EpromTrem = ");
      SerialWiFi.print(EpromTrem);      
      SerialWiFi.print(" - EpromAMVS = ");
      SerialWiFi.print(EpromAMVS);      
      SerialWiFi.println();
}
if (Caracter == 124) {                                                              // Para Copiar Dados Ativos para EEPROM digitar '|'
      SalvaTrem();
      OlharTrem();
}
if (Caracter == 64) {                                                               // Para Copiar da EEPROM para Dados Ativos digitar '@'
      CaptaTrem();
      OlharTrem();
}
if (Caracter == 38) {                                                               // Para Olhar Conteudo Ativo digitar '&'
      OlharTrem();
}
if (Caracter == 37) {                                                               // Para Gravar String RotaTrem para EEPROM digitar '%'
      GravStrChr();
      MostStrChr();
}
if (Caracter == 35) {                                                               // Para Captar String RotaTrem da EEPROM digitar '#'
      CaptStrChr();
      MostStrChr();
}
if (Caracter == 44) {                                                               // Para mostrar String digitar ',' 
      Leitura = "";
      for (VaiVem = 0; VaiVem < 64; VaiVem++) { 
      Valor = VaiVem + 1100;
      Leitura = Leitura + char(EEPROM.read(Valor)); 
  }
      SerialWiFi.print(" String = ");
      SerialWiFi.print(Leitura);   
      SerialWiFi.println("");
}
if (Caracter == 46){                                                                // Para gravar String de ateh 63 Bytes na EEPROM digitar '.' & 'String' 
      Caracter = ' '; 
  for (Valor = 1100; Valor < 1164; Valor++) {                                       // Limpando a String
  if (EEPROM.read(Valor != Caracter)) {
      EEPROM.write(Valor, Caracter); 
      EEPROM.commit();
  }
  }
      Leitura = "";
      VaiVem = 0; 
      Valor = 1100;
  while(SerialWiFi.available() >= 0 && VaiVem <= 63) {                              // Escrevendo String
      EEPROM.write(Valor, SerialWiFi.read());
      EEPROM.commit();
      Leitura = Leitura + char(EEPROM.read(Valor));
      SerialWiFi.print(" Conteudo Dec = ");
      SerialWiFi.print(EEPROM.read(Valor));      
      SerialWiFi.print(" Conteudo Char = ");
      SerialWiFi.println(char(EEPROM.read(Valor)));      
      VaiVem =  VaiVem + 1;
      Valor = Valor + 1;
    }
}
if (Caracter == 45) {                                                               //  Para ler a EEPROM digitar '-'
      ByteAlto = EEPROM.read(EpromAMVS + 100); 
      ByteBaxo = EEPROM.read(EpromAMVS); 
      Memoria = ByteAlto * 256 + ByteBaxo; 
      SerialWiFi.println("");
      SerialWiFi.print(" EpromAMVS = ");
      SerialWiFi.print(EpromAMVS);      
      SerialWiFi.print(" - Numero = ");
      SerialWiFi.println(Memoria);      
      Memoria = EEPROM.read(EpromTrem); 
      SerialWiFi.print(" EpromTrem = ");
      SerialWiFi.print(EpromTrem);      
      SerialWiFi.print(" - Diversos = ");
      SerialWiFi.println(Memoria);      
}
if (Caracter == 39) {                                                               // Para mostrar o Mapa da EEPROM digitar ' aspa simples 
      ShowDados(); 
}
if (Caracter == 126) {                                                              //  Para mostrar Tutor digitar '~' 
      Tutorial(); 
}
}

void Konsole() {
    if (millis() - Tempo1 >= 400){ 
      Tempo1 = millis(); 
    if (digitalRead(Botinp0) == LOW){                                               // Botao Botinp0 clikado Nivel Baixo altera Opsoes
      Xecar = Xecar + 1; if ( Xecar > 5 ) { Xecar = 1; }
    switch (Xecar) { 
      case 1:
          Opsao$ = "Trem"; 
      break;
      case 2:
          Opsao$ = "Efex";
      break;
      case 3:
          Opsao$ = "Slet"; 
      break;
      case 4: 
          Opsao$ = "Prog"; 
      break;
      case 5: 
          Opsao$ = "Memo"; 
      break;
    }
          Atual = 0; 
            TremPainel(); 
    } else {                                                                        // Botao Botinp0 em Nivel Alto mantem a Selecsao
    switch (Xecar) { 
      case 1:
          Opsao$ = "Trem"; 
            OkeyTrem(); 
      break;
      case 2:
          Opsao$ = "Efex"; 
            EfeitEspec();
      break;
      case 3:
          Opsao$ = "Slet"; 
            SeleTrem();
      break;
      case 4:
          Opsao$ = "Prog"; 
            MultiFunsao();
      break;
      case 5: 
          Opsao$ = "Memo"; 
          if (digitalRead(Botinp5) == LOW){                                         // Salva os Dados na EEPROM 
              Atual = 5;  Vizor$ = "*_*"; 
              SalvaTrem(); 
              TremPainel(); 
          }
          if (digitalRead(Botinp6) == LOW){ 
              Atual = 5;  Vizor$ = "^->"; 
          if (AMVSouTrem == "Trem") {                                               // Seta incremento EpromTrem na EEPROM para Grupo de Trens e Efeitos
          if (EpromTrem < 90) { EpromTrem = EpromTrem + 10;
          } else { 
            EpromTrem = 90;
          }
          } else {                                                                  // Seta incremento EpromAMVS na EEPROM para Grupo AMV Graus Semafaros Etc
          if (EpromAMVS < 80) { EpromAMVS = EpromAMVS + 16;
          } else { 
            EpromAMVS = 80;
          }
          }
              TremPainel(); 
          }
          if (digitalRead(Botinp4) == LOW){ 
              Atual = 5;  Vizor$ = "<-^"; 
          if (AMVSouTrem == "Trem") {                                               // Seta decremento EpromTrem na EEPROM para Grupo de Trens e Efeitos
          if (EpromTrem > 9) { EpromTrem = EpromTrem - 10;
          } else { 
            EpromTrem = 0;
          }
          } else {                                                                  // Seta decremento EpromAMVS na EEPROM para Grupo AMV Graus Semafaros Etc
          if (EpromAMVS > 15) { EpromAMVS = EpromAMVS - 16;
          } else { 
            EpromAMVS = 0;
          }
          }
              TremPainel(); 
          }
      break;
    }
  }
 }
}

void SeletorAMV() { 
      Extra$ = Ativa$;  Numero = Ativa$.length(); 
      Testar = Numero - 3; 
      Kodin$ = Ativa$.substring(3, Testar);                                         // Aponta para 
      SeletSM = Kodin$.toInt();
      Testar = Numero - 2; Numero = Numero -1; 
      Kodin$ = Ativa$.substring(Testar, Numero);                                    // Aponta para atual Sentido do AMV
      if (Kodin$ == "0") {
      ServoLeds.setPWM(SeletSM, 0, BasiqSM[SeletSM]);                               // Envia Sinal para o Canal do PCA9685 selecioado  
      AtualSM[SeletSM] = BasiqSM[SeletSM]; 
      RotaTrem.setCharAt(SeletSM, '|');
      } else {
      ServoLeds.setPWM(SeletSM, 0, LimitSM[SeletSM]);                               // Envia Sinal para o Canal do PCA9685 selecioado 
      AtualSM[SeletSM] = LimitSM[SeletSM]; 
      RotaTrem.setCharAt(SeletSM, 'Y');
      }
        TremPainel(); 
}

void MultiFunsao() {                                                                // Servo Motores do Canal 0 ao Canal 15
    if (digitalRead(Botinp5) == LOW){                                               // Luz e AMV Angulo Fixo ou ServoMotor Angulo Especifico
      Atual = 5; 
      if (AMVSM$ == "AMV") { AMVSM$ = "SMT"; AMVSouTrem = "AMVS"; } else { AMVSM$ = "AMV"; AMVSouTrem = "Trem"; } 
        TremPainel(); 
    } 
    if (digitalRead(Botinp6) == LOW){                                                         // Seleciona Canal Anterior Ciclo Continuo 
      Atual = 6; Panel$ = "}->";                                                              // Relativos a Central DCC 
      if (SeletSM < 15) { SeletSM = SeletSM + 1; } else if (SeletSM >= 15) { SeletSM = 0; }   // Caso Canal seja o 15 entao seleciona o Canal 0
        TremPainel(); 
    } 
    if (digitalRead(Botinp4) == LOW){                                                         // Seleciona Canal Seguinte Ciclo Continuo
      Atual = 4; Panel$ = "<-{";                                                              // Relativos a Central DCC 
      if (SeletSM > 0) { SeletSM = SeletSM - 1; } else if (SeletSM <= 0) { SeletSM = 15; }    // Caso o Canal seja o 0 entao seleciona o Canal 15
        TremPainel(); 
    } 
    if (AMVSM$ == "SMT") {
      if (RotaTrem.charAt(SeletSM) == 'Y'){
      if (digitalRead(Botinp2) == LOW){ 
        Atual = 2; 
        LimitSM[SeletSM] = AtualSM[SeletSM]; 
      if (LimitSM[SeletSM] < 1080 - Graus){ 
        LimitSM[SeletSM] = LimitSM[SeletSM] + Graus; 
        } else { 
        LimitSM[SeletSM] = 0; 
        }
        AtualSM[SeletSM] = LimitSM[SeletSM]; 
        TremPainel(); 
        }
      if (digitalRead(Botinp8) == LOW){ 
        Atual = 8; 
        LimitSM[SeletSM] = AtualSM[SeletSM]; 
      if (LimitSM[SeletSM] > Graus){ 
        LimitSM[SeletSM] = LimitSM[SeletSM] - Graus; 
        } else {
        LimitSM[SeletSM] = 1080; 
        }
        AtualSM[SeletSM] = LimitSM[SeletSM]; 
        TremPainel(); 
        }          
        } else { 
      if (digitalRead(Botinp2) == LOW){ 
        Atual = 2; 
        BasiqSM[SeletSM] = AtualSM[SeletSM]; 
      if (BasiqSM[SeletSM] < 1080 - Graus){ 
        BasiqSM[SeletSM] = BasiqSM[SeletSM] + Graus; 
        } else { 
        BasiqSM[SeletSM] = 0; 
        }
        AtualSM[SeletSM] = BasiqSM[SeletSM]; 
        TremPainel(); 
        }
      if (digitalRead(Botinp8) == LOW){ 
        Atual = 8; 
        BasiqSM[SeletSM] = AtualSM[SeletSM]; 
      if (BasiqSM[SeletSM] > Graus){ 
        BasiqSM[SeletSM] = BasiqSM[SeletSM] - Graus; 
        } else {
        BasiqSM[SeletSM] = 1080; 
        }
        AtualSM[SeletSM] = BasiqSM[SeletSM]; 
        TremPainel(); 
        }          
        } 
        } else { 
      if (RotaTrem.charAt(SeletSM) == '|'){
      if (digitalRead(Botinp2) == LOW){ 
        Atual = 2;                                                                  // Relativos a Central DCC 
        RotaTrem.setCharAt(SeletSM, 'Y');
        AtualSM[SeletSM] = 360; 
        ServoLeds.setPWM(SeletSM, RampaSM[SeletSM], AtualSM[SeletSM]);              // Envia Sinal para o Canal do PCA9685 selecioado  
        TremPainel(); 
      } 
      } else { 
      if (digitalRead(Botinp8) == LOW){ 
        Atual = 8;                                                                  // Relativos a Central DCC 
        RotaTrem.setCharAt(SeletSM, '|');
        AtualSM[SeletSM] = 0; 
        ServoLeds.setPWM(SeletSM, RampaSM[SeletSM], AtualSM[SeletSM]);              // Envia Sinal para o Canal do PCA9685 selecioado 
        TremPainel(); 
        }
      }
    }
    if (digitalRead(Botinp3) == HIGH){                                              // Opsao de Direcsao de Servo e Led
        Atual = 3;                                                                  // Relativos a Central DCC 
        LedServo(); 
        TremPainel(); 
    } 
}

void OkeyTrem(){ 
    if (Atual != 5) { 
    Testar = map(analogRead(A0),0,1023,0,99);                                       // Potenciometro Controlador da Velocidade da Locomotiva
    if (Testar != VeLoco[QualTrem]) { 
      VeLoco[QualTrem] = Testar;
      Bytes = FunLoc[QualTrem];
      LocomAtiv(); 
      LuzAud();
      SerialWiFi.println(Legal$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
      } 
    }
    if (Atual != 8 && digitalRead(Botinp2) == LOW){                                 // Avansa a Locomotiva
      Atual = 2; Vizor$ = "<+>"; 
      Acsao$ = "1"; LayKa$.setCharAt(QualTrem, '1'); 
      Bytes = FunLoc[QualTrem];
      LocomAtiv(); 
      LuzAud();
      SerialWiFi.println(Legal$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
      }
    if (digitalRead(Botinp5) == LOW){                                               // Para a Locomotiva
      Atual = 5; Vizor$ = "|!|"; 
      Bytes = FunLoc[QualTrem];
      LokomPara();                                                                  // NumLoc[10] Locomotiva Exclusiva do Painel de Controle
      LuzAud();
      SerialWiFi.println(Legal$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
      }
    if (Atual != 2 && digitalRead(Botinp8) == LOW){                                 // Recua a Locomotiva
      Atual = 8; Vizor$ = ">-<"; 
      Acsao$ = "0"; LayKa$.setCharAt(QualTrem, '0'); 
      Bytes = FunLoc[QualTrem];
      LocomAtiv(); 
      LuzAud();
      SerialWiFi.println(Legal$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
     }
    if (digitalRead(Botinp6) == LOW){ 
      Atual = 5; Panel$ = "}->"; 
      if (QualTrem < 9) { QualTrem = QualTrem + 1; } else { QualTrem = 0; }         // Ramal
      Acsao$ = LayKa$.charAt(QualTrem); 
      Bytes = FunLoc[QualTrem];
      LocomAtiv();                                                                  // NumLoc[10] Locomotiva Exclusiva do Painel de Controle
      LuzAud();
      SerialWiFi.println(Legal$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
    } 
    if (digitalRead(Botinp4) == LOW){ 
      Atual = 5; Panel$ = "<-{"; 
      if (QualTrem > 0) { QualTrem = QualTrem - 1; } else { QualTrem = 9; }         // Ramal 
      Acsao$ = LayKa$.charAt(QualTrem); 
      Bytes = FunLoc[QualTrem];
      LocomAtiv();                                                                  // NumLoc[10] Locomotiva Exclusiva do Painel de Controle
      LuzAud();
      SerialWiFi.println(Legal$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
    }
    if (digitalRead(Botinp3) == HIGH){                                              //  Opsao de Direcsao de Servo e Led
      Bytes = FunLoc[QualTrem];
      LedServo(); 
      LuzAud();
    } 
}

void SeleTrem(){ 
    if (digitalRead(Botinp2) == LOW){ 
      Atual = 5; Vizor$ = ")O("; 
      LokomPara(); 
        Final = Legal$.length();                                                    //  Tamanho da String de Comando Legal$ 
        Kodin$ = Legal$.substring(5, Final);                                        //  Pega o Numero da Locomotiva 
        Entre = Kodin$.toInt() + 10000;                                             //  Converte String do Numero em inteiro e adequa ele para VaiVems
        Kodin$ = String(Entre);                                                     //  Converte o Numero numa String 
        Tecla = Kodin$.charAt(Ponto);                                               //  Capta um Algarismo da String 
        if (Tecla < '9') { Tecla = Tecla + 1; } else { Tecla = '0'; }               //  incrementa Valor do Algarismo apontado
        Kodin$.setCharAt(Ponto, Tecla);                                             //  Grava Algarismo Atualizado na String  
        Entre = Kodin$.toInt() - 10000;                                             //  Reconverte String do Numero em inteiro e atualiza ele 
      Bytes = FunLoc[QualTrem];
      XekAcsao();
      LuzAud();
      } 
    if (digitalRead(Botinp8) == LOW){ 
      Atual = 5; Vizor$ = ")O("; 
      LokomPara(); 
        Final = Legal$.length();                                                    //  Tamanho da String de Comando Legal$ 
        Kodin$ = Legal$.substring(5, Final);                                        //  Pega o Numero da Locomotiva 
        Entre = Kodin$.toInt() + 10000;                                             //  Converte String do Numero em inteiro e adequa ele para VaiVems
        Kodin$ = String(Entre);                                                     //  Converte o Numero numa String 
        Tecla = Kodin$.charAt(Ponto);                                               //  Capta um Algarismo da String 
        if (Tecla > '0') { Tecla = Tecla - 1; } else { Tecla = '9'; }               //  incrementa Valor do Algarismo apontado
        Kodin$.setCharAt(Ponto, Tecla);                                             //  Grava Algarismo Atualizado na String  
        Entre = Kodin$.toInt() - 10000;                                             //  Reconverte String do Numero em inteiro e atualiza ele 
      Bytes = FunLoc[QualTrem];
      XekAcsao();
      LuzAud();
      } 
    if (digitalRead(Botinp5) == LOW){ 
      Atual = 5; Vizor$ = "|!|"; 
      Bytes = FunLoc[QualTrem];
      LokomPara();                                                                  // NumLoc[10] Locomotiva Exclusiva do Painel de Controle
      LuzAud();
      SerialWiFi.println(Legal$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
      } 
    if (digitalRead(Botinp6) == LOW){ 
      Atual = 5; Vizor$ = ")O("; Panel$ = "}->"; 
      if (QualTrem < 9) { QualTrem = QualTrem + 1; } else { QualTrem = 0; }         // Ramal
      Bytes = FunLoc[QualTrem];
      LokomPara(); 
      LuzAud();
      } 
    if (digitalRead(Botinp4) == LOW){ 
      Atual = 5; Vizor$ = ")O("; Panel$ = "<-{"; 
      if (QualTrem > 0) { QualTrem = QualTrem - 1; } else { QualTrem = 9; }         // Ramal
      Bytes = FunLoc[QualTrem];
      LokomPara(); 
      LuzAud();
      } 
    if (digitalRead(Botinp3) == HIGH){ 
      Atual = 3; Vizor$ = ")O("; 
      if (Ponto < 4) { Ponto = Ponto + 1; } else { Ponto = 1; }                     // Digito do Numero da Locomotiva 
      Bytes = FunLoc[QualTrem];
      LuzAud();
    }
} 

void EfeitEspec() {                    // Opsao 2                                   // Byte WeMosMini esta com apenas cinco bits por falta de pinos 
  if (digitalRead(Botinp2) == LOW){ 
    switch (Ycogn) { 
      case 0: Ycogn = 16; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 0); break;
      case 16: Ycogn = 32; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 0); break;
      case 32: Ycogn = 48; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 0); break;
      case 48: Ycogn = 64; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 0); break;
      case 64: Ycogn = 80; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 0); break;
      case 80: Ycogn = 96; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 0); break;
      case 96: Ycogn = 112; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 0); break;
      case 112: Ycogn = 128; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 0); break;
      case 128: Ycogn = 144; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 1); break;
      case 144: Ycogn = 160; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 1); break;
      case 160: Ycogn = 176; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 1); break;
      case 176: Ycogn = 192; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 0); bitWrite(Bytes, 7, 1); break;
      case 192: Ycogn = 208; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 1); break;
      case 208: Ycogn = 224; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 0); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 1); break;
      case 224: Ycogn = 240; bitWrite(Bytes, 4, 0); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 1); break;
      case 240: Ycogn = 0; bitWrite(Bytes, 4, 1); bitWrite(Bytes, 5, 1); bitWrite(Bytes, 6, 1); bitWrite(Bytes, 7, 1); break;
    } SomLuz(); return; } 
  if (digitalRead(Botinp5) == LOW){ if (bitRead(Bytes, 3) != 0) { bitWrite(Bytes, 3, 0); } else { bitWrite(Bytes, 3, 1); } SomLuz(); return; } 
  if (digitalRead(Botinp8) == LOW){ if (bitRead(Bytes, 2) != 0) { bitWrite(Bytes, 2, 0); } else { bitWrite(Bytes, 2, 1); } SomLuz(); return; } 
  if (digitalRead(Botinp6) == LOW){ if (bitRead(Bytes, 1) != 0) { bitWrite(Bytes, 1, 0); } else { bitWrite(Bytes, 1, 1); } SomLuz(); return; } 
  if (digitalRead(Botinp4) == LOW){ if (bitRead(Bytes, 0) != 0) { bitWrite(Bytes, 0, 0); } else { bitWrite(Bytes, 0, 1); } SomLuz(); return; } 
  if (digitalRead(Botinp3) == HIGH){ Xecar = 1; Opsao$ = "Trem"; TremPainel(); }
}

void XekAcsao() {
      Acsao$ = LayKa$.charAt(QualTrem); 
      MeuTrem = String(Entre);                                                      // Converte Numero da Locomotiva int em String
      NumLoc[QualTrem] = Entre; 
      LokomPara();                                                                  // NumLoc[0] Locomotiva Exclusiva do Painel de Controle
}

void LocomAtiv() { 
      Ramal$ = String(QualTrem); 
      MeuTrem = String(NumLoc[QualTrem]); 
      Veloz$ = String(VeLoco[QualTrem]); 
      LuzSom = String(FunLoc[QualTrem]); 
      Legal$ = "<t " + Ramal$ + " " + MeuTrem + " " + Veloz$ + " " + Acsao$ + ">";   // NumLoc[10] Locomotiva Exclusiva do Painel de Controle  
}

void LokomPara() { 
      Ramal$ = String(QualTrem); 
      MeuTrem = String(NumLoc[QualTrem]); 
      Veloz$ = String("0");                                                         // Velocidade "0" Zero Locomotiva "Para"
      LuzSom = String(FunLoc[QualTrem]); 
      Legal$ = "<t " + Ramal$ + " " + MeuTrem + " " + Veloz$ + " " + Acsao$ + ">";  // NumLoc[10] Locomotiva Exclusiva do Painel de Controle  
}

void SomLuz() {                                                                     // Funsoes 
      FunLoc[QualTrem] = Bytes;
      LuzSom = String(Bytes);
      BitVe$ = "0000000" + String(Bytes, BIN); 
      Numero = BitVe$.length() - 8; 
      BitVe$ = BitVe$.substring(Numero, BitVe$.length()); 
      Efeix$ = "<f " + MeuTrem + " " + LuzSom + ">"; 
      SerialWiFi.println(Efeix$); SerialWiFi.flush();                               // Coordena Comando para Central DCC
      TremPainel(); 
} 

void LuzAud() {                                                                     // Funsoes 
      FunLoc[QualTrem] = Bytes;
      LuzSom = String(Bytes);
      BitVe$ = "0000000" + String(Bytes, BIN); 
      Numero = BitVe$.length() - 8; 
      BitVe$ = BitVe$.substring(Numero, BitVe$.length()); 
      Efeix$ = "<f " + MeuTrem + " " + LuzSom + ">"; 
      TremPainel(); 
} 

void LedServo() { 
      if (AMVSM$ == "AMV") {
      Kodin$ = String(SeletSM);
      if (RotaTrem.charAt(SeletSM) == 'Y'){
      Extra$ = "<y " + Kodin$ + " 0>";                                              // Coordena Comando para Central DCC
      ServoLeds.setPWM(SeletSM, 0, 0);                                              // Envia Sinal para o Canal do PCA9685 selecioado  
      AtualSM[SeletSM] = 0; 
      RotaTrem.setCharAt(SeletSM, '|'); 
      } else { 
      Extra$ = "<y " + Kodin$ + " 1>";                                              // Coordena Comando para Central DCC
      ServoLeds.setPWM(SeletSM, 0, 360);                                            // Envia Sinal para o Canal do PCA9685 selecioado 
      AtualSM[SeletSM] = 360; 
      RotaTrem.setCharAt(SeletSM, 'Y');
      }
      Atual = 3; 
      } else { 
      Kodin$ = String(SeletSM);
      if (RotaTrem.charAt(SeletSM) == 'Y'){
      Extra$ = "<y " + Kodin$ + " 0>";                                              // Coordena Comando para Central DCC
      ServoLeds.setPWM(SeletSM, 0, BasiqSM[SeletSM]);                               // Envia Sinal para o Canal do PCA9685 selecioado  
      AtualSM[SeletSM] = BasiqSM[SeletSM]; 
      RotaTrem.setCharAt(SeletSM, '|');
      } else {
      Extra$ = "<y " + Kodin$ + " 1>";                                              // Coordena Comando para Central DCC
      ServoLeds.setPWM(SeletSM, 0, LimitSM[SeletSM]);                               // Envia Sinal para o Canal do PCA9685 selecioado 
      AtualSM[SeletSM] = LimitSM[SeletSM]; 
      RotaTrem.setCharAt(SeletSM, 'Y');
      }
      Atual = 3; 
      }
}

void TremPainel() { 
    lcd.setCursor(0,0); lcd.print("                    "); 
    lcd.setCursor(0,0); lcd.print(Opsao$); 
    lcd.setCursor(7,0); lcd.print(Vizor$); 
    lcd.setCursor(11,0); lcd.print(RotaTrem.charAt(SeletSM));                       // Ramal da Linha de Trem selecionada 
    lcd.setCursor(13,0); lcd.print(Panel$); 
    lcd.setCursor(17,0); lcd.print(AMVSM$); 
    lcd.setCursor(0,1); lcd.print("                 "); 
    lcd.setCursor(0,1); 
    if (Opsao$ == "Efex") { lcd.print(Efeix$); } else { lcd.print(Legal$); }
    lcd.setCursor(17,1); lcd.print("  "); 
    lcd.setCursor(18,1); lcd.print(Ponto); 
    lcd.setCursor(0,2); lcd.print("          "); 
    lcd.setCursor(0,2); lcd.print(Extra$); 
    lcd.setCursor(10,2); lcd.print("B         "); 
    lcd.setCursor(12,2); lcd.print(BitVe$); 
    lcd.setCursor(0,3); lcd.print("Setor               "); 
    lcd.setCursor(6,3); lcd.print(SeletSM);                                         // Canal Ativo
    lcd.setCursor(10,3); lcd.print("Graus"); 
    lcd.setCursor(16,3); lcd.print(AtualSM[SeletSM]);                               // Grau Atual referente ao Canal selecionado 
} 

void CaptStrChr() {                                                                 // Captar RotaTrem 
      ZigZag = "";
  for (VaiVem = 0; VaiVem < 16; VaiVem++) { 
      Valor = 1000 + VaiVem;
      ZigZag = ZigZag + char(EEPROM.read(Valor)); 
  }
      StriChar = ZigZag;
}

void GravStrChr(){                                                                  // Gravar RotaTrem 
      StriChar.toCharArray(Palavra, 17);
  for (VaiVem = 0; VaiVem < 16; VaiVem++) { 
      Valor = 1000 + VaiVem;
  if (EEPROM.read(Valor != Palavra[VaiVem])) {
      EEPROM.write(Valor, Palavra[VaiVem]); 
      EEPROM.commit();
    }
  }
}

void MostStrChr() {                                                                 // Mostrar RotaTrem
      StriChar.toCharArray(Palavra, 17);
      SerialWiFi.print(" - StriChar = ");
      SerialWiFi.print(StriChar);
      SerialWiFi.println();
  for (VaiVem = 0; VaiVem < 16; VaiVem++) { 
      Valor = 1000 + VaiVem;
      SerialWiFi.print(" - Local = ");
      SerialWiFi.print(Valor);
      SerialWiFi.print(" - Palavra[] = ");
      SerialWiFi.print(Palavra[VaiVem]);
      SerialWiFi.println();
  }
      SerialWiFi.print(" - Palavra = ");
      SerialWiFi.print(Palavra);
      SerialWiFi.println();
}

void CaptaTrem() {
  for (VaiVem = 0; VaiVem < 10; VaiVem++) {                                         // Ramal
      ByteAlto = EEPROM.read(EpromAMVS + VaiVem + 100);                             // Capta Valor Maior no enderecso Alto da EEPROM
      ByteBaxo = EEPROM.read(EpromAMVS + VaiVem);                                   // Capta Valor Menor no enderecso Baixo da EEPROM
      NumLoc[VaiVem] = ByteAlto * 256 + ByteBaxo;                                   // Numero da Locomotiva
      FunLoc[VaiVem] = EEPROM.read(EpromAMVS + VaiVem + 200);                       // Efeitos Luz Som
//      Yyyyyyy[VaiVem] = EEPROM.read(EpromAMVS + VaiVem + 300);                    // Yyyyyyy
//      Zzzzzzz[VaiVem] = EEPROM.read(EpromAMVS + VaiVem + 400);                    // Zzzzzzz
  }    
  for (VaiVem = 0; VaiVem < 16; VaiVem++) {                                         // AMV SMT
      AtualSM[VaiVem] = EEPROM.read(SMTouAMV + VaiVem + 500);
      BasiqSM[VaiVem] = EEPROM.read(SMTouAMV + VaiVem + 600);
      LimitSM[VaiVem] = EEPROM.read(SMTouAMV + VaiVem + 700);
      RampaSM[VaiVem] = EEPROM.read(SMTouAMV + VaiVem + 800);
      RumosSM[VaiVem] = EEPROM.read(SMTouAMV + VaiVem + 900);
      EpromTrem = EEPROM.read(999);
      EpromAMVS = EEPROM.read(998);
  }    
}

void SalvaTrem() {
  for (VaiVem = 0; VaiVem < 10; VaiVem++) { 
      ByteAlto = NumLoc[VaiVem] / 256; 
      ByteBaxo = NumLoc[VaiVem]; 
  if (EEPROM.read((EpromAMVS + VaiVem) != ByteBaxo)) {EEPROM.write((EpromAMVS + VaiVem), ByteBaxo); EEPROM.commit();}
  if (EEPROM.read((EpromAMVS + VaiVem + 100) != ByteAlto)) {EEPROM.write((EpromAMVS + VaiVem + 100), ByteAlto); EEPROM.commit();}
  if (EEPROM.read((EpromAMVS + VaiVem + 200) != FunLoc[VaiVem])) {EEPROM.write((EpromAMVS + VaiVem + 200), FunLoc[VaiVem]); EEPROM.commit();}
//  if (EEPROM.read((EpromAMVS + VaiVem + 300) != Yyyyyyy[VaiVem])) {EEPROM.write((EpromAMVS + VaiVem + 300), Yyyyyyy[VaiVem]); EEPROM.commit();}
//  if (EEPROM.read((EpromAMVS + VaiVem + 400) != Zzzzzzz[VaiVem])) {EEPROM.write((EpromAMVS + VaiVem + 400), Zzzzzzz[VaiVem]); EEPROM.commit();}
  }
  for (VaiVem = 0; VaiVem < 16; VaiVem++) {                         // AMV SMT
  if (EEPROM.read((SMTouAMV + VaiVem + 500) != AtualSM[VaiVem])) {EEPROM.write((SMTouAMV + VaiVem + 500), AtualSM[VaiVem]); EEPROM.commit();}
  if (EEPROM.read((SMTouAMV + VaiVem + 600) != BasiqSM[VaiVem])) {EEPROM.write((SMTouAMV + VaiVem + 600), BasiqSM[VaiVem]); EEPROM.commit();}
  if (EEPROM.read((SMTouAMV + VaiVem + 700) != LimitSM[VaiVem])) {EEPROM.write((SMTouAMV + VaiVem + 700), LimitSM[VaiVem]); EEPROM.commit();}
  if (EEPROM.read((SMTouAMV + VaiVem + 800) != RampaSM[VaiVem])) {EEPROM.write((SMTouAMV + VaiVem + 800), RampaSM[VaiVem]); EEPROM.commit();}
  if (EEPROM.read((SMTouAMV + VaiVem + 900) != RumosSM[VaiVem])) {EEPROM.write((SMTouAMV + VaiVem + 900), RumosSM[VaiVem]); EEPROM.commit();}
  }    
  if (EEPROM.read(999 != EpromTrem)) {EEPROM.write(999, EpromTrem); EEPROM.commit();} // Guarda Posicsao da EEPROM para Grupo de Trens e Efeitos
  if (EEPROM.read(998 != EpromAMVS)) {EEPROM.write(998, EpromAMVS); EEPROM.commit();} // Guarda Posicsao da EEPROM para AMV Graus Semafaros Etc
 }

void OlharTrem() {                                                  //  Mostra Conteudo Ativo
  for (VaiVem = 0; VaiVem < 10; VaiVem++) { 
      ByteAlto = NumLoc[VaiVem] / 256; 
      ByteBaxo = NumLoc[VaiVem]; 
      SerialWiFi.print(" Ramal = ");
      SerialWiFi.print(VaiVem);      
      SerialWiFi.print(" - Trem = ");
      SerialWiFi.print(ByteAlto * 256 + ByteBaxo);      
      SerialWiFi.print(" - Velocidade = ");
      SerialWiFi.print(VeLoco[VaiVem]);      
      SerialWiFi.print(" - Movimento = ");
      SerialWiFi.print(QuePAR[VaiVem]);      
      SerialWiFi.print(" - SomLuz = ");
      Leitura = "00000000" + String(FunLoc[VaiVem], BIN);           //  Complementa string Conversa com Zeros a Esquerda
      Numero = Leitura.length() - 8; 
      Leitura = Leitura.substring(Numero, Leitura.length());        //  Adequa string Conversa afim dela se manter sempre com Dez Caracteres
      SerialWiFi.print(Leitura);      
      SerialWiFi.print(" < Bin - ");
      SerialWiFi.print(FunLoc[VaiVem], HEX);      
      SerialWiFi.print(" < Hex - ");
      SerialWiFi.print(FunLoc[VaiVem], DEC);      
      SerialWiFi.print(" < Dec - ");
      SerialWiFi.println("");
  }
  for (VaiVem = 0; VaiVem < 16; VaiVem++) {                         // AMV SMT
      SerialWiFi.print(" AMV SMT = ");
      SerialWiFi.print(VaiVem);
      SerialWiFi.print(" AtualSM = ");
      SerialWiFi.print(AtualSM[VaiVem]);
      SerialWiFi.print(" BasiqSM = ");
      SerialWiFi.print(BasiqSM[VaiVem]);
      SerialWiFi.print(" LimitSM = ");
      SerialWiFi.print(LimitSM[VaiVem]);
      SerialWiFi.print(" RampaSM = ");
      SerialWiFi.print(RampaSM[VaiVem]);
      SerialWiFi.print(" RumosSM = ");
      SerialWiFi.print(RumosSM[VaiVem]);
      SerialWiFi.println("");
  } 
      SerialWiFi.print(" - StriChar = ");
      SerialWiFi.print(StriChar);
      SerialWiFi.println();
      SerialWiFi.print(" - EpromAMVS = ");
      SerialWiFi.print(EpromAMVS);      
      SerialWiFi.print(" - EpromTrem = ");
      SerialWiFi.print(EpromTrem);      
      SerialWiFi.print(" - SMTouAMV = ");
      SerialWiFi.print(SMTouAMV);      
      SerialWiFi.println("");
      SerialWiFi.println("");
}

void ShowDados() {                                                  //  Mostra Conteudo da EEPROM
  for (VaiVem = 0; VaiVem < 100; VaiVem++) { 
      ByteAlto = EEPROM.read(VaiVem + 100); 
      ByteBaxo = EEPROM.read(VaiVem); 
      SerialWiFi.print(" Memoria = ");
      SerialWiFi.print(VaiVem);      
      SerialWiFi.print(" - Trem = ");
      SerialWiFi.print(ByteAlto * 256 + ByteBaxo);      
//      SerialWiFi.print(" - Zzzzzzz = ");
//      SerialWiFi.print(EEPROM.read(VaiVem + 400));      
//      SerialWiFi.print(" - Yyyyyyy = ");
//      SerialWiFi.print(EEPROM.read(VaiVem + 300));      
      SerialWiFi.print(" - LuzSom = ");
      Leitura = "00000000" + String(EEPROM.read(VaiVem + 200), BIN);  //  Complementa string Conversa com Zeros a Esquerda
      Numero = Leitura.length() - 8; 
      Leitura = Leitura.substring(Numero, Leitura.length());        //  Adequa string Conversa afim dela se manter sempre com Dez Caracteres
      SerialWiFi.print(Leitura);      
      SerialWiFi.print(" < Bin - ");
      SerialWiFi.print(EEPROM.read(VaiVem + 200), HEX);      
      SerialWiFi.print(" < Hex - ");
      SerialWiFi.print(EEPROM.read(VaiVem + 200), DEC);      
      SerialWiFi.print(" < Dec - ");
      SerialWiFi.println("");
  }    
      SerialWiFi.println("");
}

void Mensagem() {
    lcd.begin(20, 4);                                                               // Mostra informacoes no display
    lcd.setCursor(5,0); lcd.print("Joahermen");
    lcd.setCursor(3,1); lcd.print("Tribo Arduino");
    lcd.setCursor(4,2); lcd.print("Sistema");
    lcd.setCursor(8,3); lcd.print("RealJoia");
}

void Tutorial() { 
      SerialWiFi.println("");
      SerialWiFi.println(" Uno EEPROM Teclado "); 
      SerialWiFi.println("              Autor Joahermen "); 
      SerialWiFi.println(" Para Selecionar EpromTrem da EEPROM para Gravar e Ler digitar ';' & 'Localizacsao' "); 
      SerialWiFi.println(" Exemplo para selecionar Local 19 da Memoria ;19 ");
      SerialWiFi.println(" Para gravar na EEPROM digitar ':' & 'Numero de 2 Digitos e maximo de 300 Locais' "); 
      SerialWiFi.println(" Exemplo para gravar o Numero 89 na Memoria :89 ");
      SerialWiFi.println(" Para gravar na EEPROM digitar '=' & 'Numero de 4 Digitos e maximo de 100 Locais' "); 
      SerialWiFi.println(" Exemplo para gravar o Numero 6958 na Memoria =6958 ");
      SerialWiFi.println(" Para ler o atual Local Setado da EEPROM digitar '-' "); 
      SerialWiFi.println(" Para Setar EpromTrem ou EpromAMVS incrementando Bloco da EEPROM digitar '>' "); 
      SerialWiFi.println(" Para Setar EpromTrem ou EpromAMVS Decrementando Bloco da EEPROM digitar '<' "); 
      SerialWiFi.println(" Para gravar String de ateh 63 Bytes na EEPROM digitar '.' & 'String' "); 
      SerialWiFi.println(" Exemplo para gravar o String RealJoia na Memoria .RealJoia ");
      SerialWiFi.println(" Para mostrar String digitar ',' "); 
      SerialWiFi.println(" Para Copiar Dados ativos para a EEPROM digitar '|' "); 
      SerialWiFi.println(" Para Copiar da EEPROM para Dados Ativos digitar '@' ");
      SerialWiFi.println(" Para Olhar Conteudo Ativo digitar '&' ");
      SerialWiFi.println(" Para Gravar String RotaTrem para EEPROM digitar '%' "); 
      SerialWiFi.println(" Para Captar String RotaTrem da EEPROM digitar '#' ");
      SerialWiFi.println(" Para mostrar o Mapa da EEPROM digitar ' aspa simples "); 
      SerialWiFi.println(" Para Zerar todos Locais da EEPROM digitar '^' ! ATENSAO - CUIDADO !"); 
      SerialWiFi.println(" Para mostrar esse Tutor novamente digite '~' ");
      SerialWiFi.println("");
}

/*
		Painel Central DCC RealJoia 
 
 Posicsao e Cores dos Botoes no Painel da Central DCC RealJoia
 
                < Vd >                      Vd = Verde
                                            Br = Branco
        < Br >  < Vm >  < Az >              Vm = Vermelho
                                            Az = Azul
                < Am >                      Am = Amarelo
                                            Pt = Preto
                < Pt >  < Bc >              Bc = Branco

Mostrador  Trem                Efex                Slet                Prog                Memo
Botoes      Atividade           Efeito              Numero              AMV SMT             EEPROM
 < Vd >     Avante              inc Bits 4-7        increNumer          inc Valores
 < Br >     DecSelTrem          Seta Bit 0          DecSelTrem          DecSetor 15 a 0
 < Vm >     Parar               Seta Bit 4          | - |               AMV <> SMT          Salva
 < Az >     incSelTrem          Seta Bit 1          incSelTrem          incSetor 0 a 15
 < Am >     Recua               Seta Bit 2          DecreNumer          Dec Valores
 < Pt >     Desvio AMV          Seleta Trem         Set Bit 1-4         Desv Princ|Sec
 < Bc >     Seleta Efex         Seleta Slet         Seleta Prog         Seleta Memo         Seleta Trem
 
 AMV = Valores Fixos de 0 ou 360 Graus 
 SMT = Valores Programaveis 

*******************************************************************************

	WeMos D1 Mini Esp 8266 - Pinos : 

		GPiO = 1  _ Tx - Nivel PullUp 
		GPiO = 3  _ Rx - Nivel PullUp 
		GPiO = 4  _ D2 - SDA - Nivel Ambos
		GPiO = 5  _ D1 - SCL - Nivel Ambos 
		GPiO = 0  _ D3 - Nivel PullUp 
		GPiO = 2  _ D4 - Nivel PullUp 
		GPiO = 12 _ D6 - Nivel Ambos 
		GPiO = 13 _ D7 - Nivel Ambos 
		GPiO = 14 _ D5 - Nivel Ambos 
		GPiO = 15 _ D8 - Nivel PullDown
		GPiO = 16 _ D0 - Nivel instavel 

*******************************************************************************

    Sentidos - Linha Principal = 0 = '|||'   -   Linha Secundaria = 1 = '|Y|'  
    Exemplo AMV numero 10 sentido Principal <Y 9 0>  
    Exemplo AMV numero 10 sentido Secundaria <Y 9 1> 
                                * * * - A T E N S A O - * * * 
    Fonte independente para os acessorios 
    Utilisar outros pinos do WeMos Mini para comandos diversos e 
    nunca usar Tx-Rx para tipo "SerialWiFi.print" exceto comandos proprios do DCC ++  
    pois os AMV e Sensores funcionam independentes da Central DDC 
    a qual deve ter o programa relativo preservado
    AMV para Reta = 1 e AMV Tanto Direita quanto Esquerda = 0 
    AMVs ateh de 1 a 121 com cerca de sete PCA9685 em cascata 
*/
//        ***   Autor Joaquim Herdy Menezes = Joahermen   *** 
//

